# Library
library(xts)

.originDate <- as.Date("1899-12-30")
.fromComDate <- function(d)
{
	as.Date(d, origin = .originDate)
	# v <- d-25569
	# class(v) <- "Date"
	# v
}

setGeneric(".fromComValueDate", function(v) v)
setMethod(".fromComValueDate", signature("COMDate"), function(v) .fromComDate(v))

# Convert to COMDate if it is a date
.startComDate <- as.POSIXlt("1899-12-30 CET")
setGeneric(".toComValue", function(d) d)
setMethod(".toComValue", signature("Date"), function(d) new("COMDate", unclass(d)+25569))
setMethod(".toComValue", signature("POSIXlt"), function(d)
{
	tzoffset = as.POSIXlt(Sys.time())$gmtoff
	seconds = as.double(d) - as.double(.startComDate) + tzoffset
	new("COMDate", seconds/86400)
})
setMethod(".toComValue", signature("list"), function(d) sapply(d, function(x) .toComValue(x)))

.fromComDateAsPOSIXlt <- function(d)
{
    seconds = unclass(d)*86400
	tzoffset = as.POSIXlt(Sys.time())$gmtoff
	as.POSIXlt(seconds - tzoffset, origin=.startComDate)
}

.nullAsEmptyString <- function(v) if (is.null(v)) "" else v

.findEnum <- function(valueList, valueToFind, functionWhenNotFound)
{
	index <- match(valueToFind, valueList, nomatch = 0)
	if (index == 0)
	   functionWhenNotFound(valueToFind)
	else
		valueList[index]
}

setGeneric(".fromComValue", function(v) v)
setMethod(".fromComValue", signature("COMDate"), function(v) .fromComDateAsPOSIXlt(v))

setClass("MacrobondDB", slots = c(connection = "COMIDispatch", db = "COMIDispatch", version = "integer"))

Entity <- setClass("Entity", slots = c(object = "COMIDispatch"))
TimeSeries <- setClass("TimeSeries", contains = "Entity")
SeriesRequest <- setClass("SeriesRequest", slots = c(object = "COMIDispatch"))

TimeSeriesFrequencyClass <- setClass("TimeSeriesFrequencyClass", contains = "integer")
TimeSeriesFrequency <- list(
   "Annual" = TimeSeriesFrequencyClass(1),
   "SemiAnnual" = TimeSeriesFrequencyClass(2),
   "QuadMonthly" = TimeSeriesFrequencyClass(3),
   "Quarterly" = TimeSeriesFrequencyClass(4),
   "BiMonthly" = TimeSeriesFrequencyClass(5),
   "Monthly" = TimeSeriesFrequencyClass(6),
   "Weekly" = TimeSeriesFrequencyClass(7),
   "Daily" = TimeSeriesFrequencyClass(8),
   "Lowest" = TimeSeriesFrequencyClass(100),
   "Highest" = TimeSeriesFrequencyClass(101))

setGeneric(".toFrequencyEnum", function (f) standardGeneric(".toFrequencyEnum"))
setMethod(".toFrequencyEnum", signature("TimeSeriesFrequencyClass"), function (f) unclass(f))
setMethod(".toFrequencyEnum", signature("character"), function (f) unclass(TimeSeriesFrequency[[f]]))

TimeSeriesWeekdaysClass <- setClass("TimeSeriesWeekdaysClass", contains = "integer")
TimeSeriesWeekdays <- list(
   "Friday" = TimeSeriesWeekdaysClass(32),
"FullWeek" = TimeSeriesWeekdaysClass(127),
"Monday"= TimeSeriesWeekdaysClass(2),
"MondayToFriday" = TimeSeriesWeekdaysClass(62),
"MondayToThursdayAndSaturday" = TimeSeriesWeekdaysClass(94),
"Saturday" = TimeSeriesWeekdaysClass(64),
"SaturdayToThursday" = TimeSeriesWeekdaysClass(95),
"SaturdayToWednesday" = TimeSeriesWeekdaysClass(79),
"Sunday" = TimeSeriesWeekdaysClass(1),
"SundayToThursday" = TimeSeriesWeekdaysClass(31),
"Thursday" = TimeSeriesWeekdaysClass(16),
"Tuesday" = TimeSeriesWeekdaysClass(4),
"Wednesday" = TimeSeriesWeekdaysClass(8))

setGeneric(".toWeekdayEnum", function (w) standardGeneric(".toWeekdayEnum"))
setMethod(".toWeekdayEnum", signature("TimeSeriesWeekdaysClass"), function (w) unclass(w))
setMethod(".toWeekdayEnum", signature("character"), function (w) unclass(TimeSeriesWeekdays[[w]]))

CalendarMergeModeClass <- setClass("CalendarMergeModeClass", contains = "integer")
CalendarMergeMode <- list(
   "FullCalendar" = CalendarMergeModeClass(0),
   "AvailableInAll" = CalendarMergeModeClass(1),
   "AvailableInAny" = CalendarMergeModeClass(2))

CalendarDateModeClass <- setClass("CalendarDateModeClass", contains = "integer")
CalendarDateMode <- list(
   "DataInAnySeries" = CalendarDateModeClass(0),
   "DataInAllSeries" = CalendarDateModeClass(1))

TimeSeriesMissingValueMethodClass <- setClass("TimeSeriesMissingValueMethodClass", contains = "integer")
TimeSeriesMissingValueMethod <- list(
   "None" = TimeSeriesMissingValueMethodClass(0),
   "Auto" = TimeSeriesMissingValueMethodClass(1),
   "PreviousValue" = TimeSeriesMissingValueMethodClass(2),
   "ZeroValue" = TimeSeriesMissingValueMethodClass(3),
   "LinearInterpolation" = TimeSeriesMissingValueMethodClass(4))

TimeSeriesToHigherFrequencyMethodClass <- setClass("TimeSeriesToHigherFrequencyMethodClass", contains = "integer")
TimeSeriesToHigherFrequencyMethod <- list(
   "Auto" = TimeSeriesToHigherFrequencyMethodClass(0),
   "Same" = TimeSeriesToHigherFrequencyMethodClass(1),
   "Distribute" = TimeSeriesToHigherFrequencyMethodClass(2),
   "PercentageChange" = TimeSeriesToHigherFrequencyMethodClass(3),
   "LinearInterpolation" = TimeSeriesToHigherFrequencyMethodClass(4),
   "Pulse" = TimeSeriesToHigherFrequencyMethodClass(5),
   "QuadraticDistribution" = TimeSeriesToHigherFrequencyMethodClass(6),
   "CubicInterpolation" = TimeSeriesToHigherFrequencyMethodClass(7),
   "ConditionalPercentageChange" = TimeSeriesToHigherFrequencyMethodClass(8))

TimeSeriesToLowerFrequencyMethodClass <- setClass("TimeSeriesToLowerFrequencyMethodClass", contains = "integer")
TimeSeriesToLowerFrequencyMethod <- list(
   "Auto" = TimeSeriesToLowerFrequencyMethodClass(0),
   "Last" = TimeSeriesToLowerFrequencyMethodClass(1),
   "First" = TimeSeriesToLowerFrequencyMethodClass(2),
   "Flow" = TimeSeriesToLowerFrequencyMethodClass(3),
   "PercentageChange" = TimeSeriesToLowerFrequencyMethodClass(4),
   "Highest" = TimeSeriesToLowerFrequencyMethodClass(5),
   "Lowest" = TimeSeriesToLowerFrequencyMethodClass(6),
   "Average" = TimeSeriesToLowerFrequencyMethodClass(7),
   "ConditionalPercentageChange" = TimeSeriesToLowerFrequencyMethodClass(8))

# Database

FetchOneTimeSeries <- function(seriesName)
{
	comSeries <- Macrobond()@db$FetchOneSeries(seriesName)
	TimeSeries(object = comSeries)
}

setGeneric("FetchTimeSeries", function(request)
{
	comSeries <- Macrobond()@db$FetchSeries(request)
	sl <- lapply(comSeries, function(s) TimeSeries(object = s))
	names(sl) <- lapply(comSeries, function(s) s[["Name"]])
	sl
})

setMethod("FetchTimeSeries", signature("SeriesRequest"), function(request) FetchTimeSeries(request@object) )

FetchOneEntity <- function(entityName)
{
	comEntity <- Macrobond()@db$FetchOneEntity(entityName)
	Entity(object = comEntity)
}

FetchEntities <- function(entityNames)
{
	comEntities <- Macrobond()@db$FetchEntities(entityNames)
	sl <- lapply(comEntities, function(s) Entity(object = s))
	names(sl) <- lapply(comEntities, function(s) s[["Name"]])
	sl
}

CreateUnifiedTimeSeriesRequest <- function()
{
	comRequest <- Macrobond()@db$CreateUnifiedSeriesRequest()
	SeriesRequest(object = comRequest)
}

CreateTimeSeriesObject <- function(name, description, region, category, frequency, startDateOrDates, values, dayMask = TimeSeriesWeekdays[["MondayToFriday"]], metadata = CreateEmptyMetadata())
{
    TimeSeries(object = Macrobond()@db$CreateSeriesObject(name, description, region, category, .toFrequencyEnum(frequency), .toWeekdayEnum(dayMask), .toComValue(startDateOrDates), values, metadata@object))
}

CreateTimeSeriesObjectWithForecastFlags <- function(name, description, region, category, frequency, startDateOrDates, values, forecastFlags, dayMask = TimeSeriesWeekdays[["MondayToFriday"]], metadata = CreateEmptyMetadata())
{
    TimeSeries(object = Macrobond()@db$CreateSeriesObjectWithForecastFlags(name, description, region, category, .toFrequencyEnum(frequency), .toWeekdayEnum(dayMask), .toComValue(startDateOrDates), values, forecastFlags, metadata@object))
}

setGeneric("UploadOneOrMoreTimeSeries", function(series) standardGeneric("UploadOneOrMoreTimeSeries"))

setMethod("UploadOneOrMoreTimeSeries", signature("TimeSeries"), function(series) invisible(Macrobond()@db$UploadOneOrMoreSeries(series@object)) )

setMethod("UploadOneOrMoreTimeSeries", signature("list"), function(series)
{
	seriesObjects <- sapply(series, function(s) s@object)
	invisible(Macrobond()@db$UploadOneOrMoreSeries(seriesObjects))
})

DeleteOneOrMoreTimeSeries <- function(name)
{
     invisible(Macrobond()@db$DeleteOneOrMoreSeries(name))
}

CreateEmptyMetadata <- function()
{
     Metadata(object = Macrobond()@db$CreateEmptyMetadata())
}

CreateDerivedMetadata <- function(metadata)
{
     Metadata(object = Macrobond()@db$CreateDerivedMetadata(metadata@object))
}

GetMetadataInformation <- function(name)
{
     MetadataInformation(object = Macrobond()@db$GetMetadataInformation(name))
}

GetMacrobondVersion <- function()
{
	Macrobond()@version;
}

# TimeSeries
setGeneric("getName", function(object) standardGeneric("getName"))
setMethod("getName", signature("Entity"), function(object) object@object[["Name"]])

setGeneric("getPrimaryName", function(object) standardGeneric("getPrimaryName"))
setMethod("getPrimaryName", signature("Entity"), function(object) object@object[["PrimaryName"]])

setGeneric("getIsError", function(series) standardGeneric("getIsError"))
setMethod("getIsError", signature("Entity"), function(series) series@object[["IsError"]])

setGeneric("getErrorMessage", function(series) standardGeneric("getErrorMessage"))
setMethod("getErrorMessage", signature("Entity"), function(series) series@object[["ErrorMessage"]])

setGeneric("getTitle", function(series) standardGeneric("getTitle"))
setMethod("getTitle", signature("Entity"), function(series) series@object[["Title"]])

setGeneric("getMetadata", function(series) standardGeneric("getMetadata"))
setMethod("getMetadata", signature("Entity"), function(series) Metadata(object = series@object[["Metadata"]]))

setGeneric("getValues", function(series) standardGeneric("getValues"))
setMethod("getValues", signature("TimeSeries"), function(series) series@object[["Values"]])

setGeneric("getDatesAtStartOfPeriod", function(series) standardGeneric("getDatesAtStartOfPeriod"))
setMethod("getDatesAtStartOfPeriod", signature("TimeSeries"), function(series) .fromComDate(series@object[["DatesAtStartOfPeriod"]]))

setGeneric("getDatesAtEndOfPeriod", function(series) standardGeneric("getDatesAtEndOfPeriod"))
setMethod("getDatesAtEndOfPeriod", signature("TimeSeries"), function(series) .fromComDate(series@object[["DatesAtEndOfPeriod"]]))

setGeneric("getForecastFlags", function(series) standardGeneric("getForecastFlags"))
setMethod("getForecastFlags", signature("TimeSeries"), function(series) series@object[["ForecastFlags"]])

setGeneric("getStartDate", function(object) standardGeneric("getStartDate"))
setMethod("getStartDate", signature("TimeSeries"), function(object) .fromComDate(object@object[["StartDate"]]))

setGeneric("getEndDate", function(object) standardGeneric("getEndDate"))
setMethod("getEndDate", signature("TimeSeries"), function(object) .fromComDate(object@object[["EndDate"]]))

setGeneric("getFrequency", function(object) standardGeneric("getFrequency"))
setMethod("getFrequency", signature("TimeSeries"), function(object) .findEnum(TimeSeriesFrequency, object@object[["Frequency"]], TimeSeriesFrequencyClass))

setGeneric("getWeekdays", function(object) standardGeneric("getWeekdays"))
setMethod("getWeekdays", signature("TimeSeries"), function(object) .findEnum(TimeSeriesWeekdays, object@object[["Weekdays"]], TimeSeriesWeekdaysClass))

setGeneric("getValueAtDate", function(series, date) standardGeneric("getValueAtDate"))
setMethod("getValueAtDate", signature("TimeSeries", "Date"), function(series, date) series@object$GetValueAtDate(.toComValue(date)))

setGeneric("getIndexAtDate", function(series, date) standardGeneric("getIndexAtDate"))
setMethod("getIndexAtDate", signature("TimeSeries", "Date"), function(series, date) series@object$GetIndexAtDate(.toComValue(date))+1)

setGeneric("getTypicalObservationCountPerYear", function(series) standardGeneric("getTypicalObservationCountPerYear"))
setMethod("getTypicalObservationCountPerYear", signature("TimeSeries"), function(series) series@object[["TypicalObservationCountPerYear"]])

.showMetadata <- function(meta, name)
{
	value <- getFirstMetadataValue(meta, name)
	if (!is.null(value))
	{
		mi <- GetMetadataInformation(name)
		text <- getValuePresentationText(mi, value)
		title <- getDescription(mi)
		
		comment <- if (getUsesValueList(mi)) getValueInformation(mi, value)[["Comment"]] else ""

		if (comment == "")
			cat(title, ":", text, "\n")
		else
			cat(title, ":", text, paste0("(", comment, ")"), "\n")
	}
}

setMethod("show", signature("TimeSeries"), function(object)
{
	cat("Macrobond series\n")

	if (!getIsError(object))
	{
		cat("Title : ", getTitle(object), "\n")
		cat("Name : ", getName(object), "\n")
		mi <- getMetadata(object)
		cat("Primary name : ", getPrimaryName(object), "\n")
		.showMetadata(mi, "Ticker")
		.showMetadata(mi, "MLRating")
		.showMetadata(mi, "MarkitPmiCode")
		.showMetadata(mi, "BubaCode")
		.showMetadata(mi, "FederalReserveCode")
		.showMetadata(mi, "OecdEOCode")
		.showMetadata(mi, "SBGermanCode")
		.showMetadata(mi, "BOECode")
		.showMetadata(mi, "EcbCode")
		.showMetadata(mi, "BisCode")
		.showMetadata(mi, "BisTable")
		.showMetadata(mi, "INSEECode")
		.showMetadata(mi, "BeaCode")
		.showMetadata(mi, "OnsCode")
	
		.showMetadata(mi, "Region")
		.showMetadata(mi, "OriginalFrequency")
		.showMetadata(mi, "LinkedInfo")
		.showMetadata(mi, "IndexBase")
		.showMetadata(mi, "Class")
		.showMetadata(mi, "DisplayUnit")
		.showMetadata(mi, "Currency")
		.showMetadata(mi, "UnderlyingCurrency")
	
		.showMetadata(mi, "InflAdj")
		.showMetadata(mi, "PriceBase")
		.showMetadata(mi, "MoveBase")
		.showMetadata(mi, "SeasonAdj")
		.showMetadata(mi, "Percent")
		.showMetadata(mi, "PriceBasis")
		.showMetadata(mi, "TimeDim")
		.showMetadata(mi, "TrendAdj")
		.showMetadata(mi, "UnderlyingInstrumentInfo")
		.showMetadata(mi, "CdsLiquidity")
	
		.showMetadata(mi, "Source")

		cat("Frequency :", names(getFrequency(object)), "\n")
		cat(paste("Start date : ", getStartDate(object), "\n"))
		cat("\nThe last 12 values:\n")
		d = rev(getDatesAtStartOfPeriod(object))[1:12]
		v = rev(getValues(object))[1:12]
		show(data.frame(Date=d, Value=v))
	}
	else
	{
		cat("Name : ", getName(object), "\n")
		cat("Error : ", getErrorMessage(object), "\n")
	}
})

setMethod("show", signature("Entity"), function(object)
{
	cat("Macrobond entity\n")

	if (!getIsError(object))
	{
		cat("Title : ", getTitle(object), "\n")
		cat("Name : ", getName(object), "\n")

		mi <- getMetadata(object)

		cat("Primary name : ", getPrimaryName(object), "\n")

		entityType <- getFirstMetadataValue(mi, "EntityType")
		if (!is.null(entityType))
			cat("Entity type : ", entityType, "\n")

		.showMetadata(mi, "Region")
		.showMetadata(mi, "Source")

		.showMetadata(mi, "LastReleaseEventTime")
		.showMetadata(mi, "NextReleaseEventTime")
	}
	else
	{
		cat("Name : ", getName(object), "\n")
		cat("Error : ", getErrorMessage(object), "\n")
	}
})

plot.TimeSeries <- function(x, ...)
{
	displayUnit <- getFirstMetadataValue(getMetadata(x), "DisplayUnit")
	if (is.null(displayUnit))
		plot(as.xts(x), main = getTitle(x), ...)
	else
		plot(as.xts(x), main = getTitle(x), ylab = displayUnit, ...)
}

# Metadata
Metadata <- setClass("Metadata", slots = c(object = "COMIDispatch"))

setGeneric("getMetadataNames", function(m) standardGeneric("getMetadataNames"))
setMethod("getMetadataNames", signature("Metadata"), function(m) { arrayOfNames <- simplify2array(m@object$ListNames()); colnames(arrayOfNames) <- c("Name", "Description"); arrayOfNames } )

setGeneric("getIsReadonly", function(m) standardGeneric("getIsReadonly"))
setMethod("getIsReadonly", signature("Metadata"), function(m) m@object[["IsReadonly"]] )

setGeneric("addMetadataValue", function(m, name, value) standardGeneric("addMetadataValue"))
setMethod("addMetadataValue", signature("Metadata", "character", "ANY"), function(m, name, value) invisible(m@object$AddValue(name, .toComValue(value))) )

setGeneric("hideMetadataValue", function(m, name) standardGeneric("hideMetadataValue"))
setMethod("hideMetadataValue", signature("Metadata", "character"), function(m, name) invisible(m@object$HideValue(name)) )

setGeneric("getFirstMetadataValue", function(m, name) standardGeneric("getFirstMetadataValue"))
setMethod("getFirstMetadataValue", signature("Metadata", "character"), function(m, name) .fromComValue(m@object$GetFirstValue(name)))

setGeneric("getMetadataValues", function(m, name) standardGeneric("getMetadataValues"))
setMethod("getMetadataValues", signature("Metadata", "character"), function(m, name) lapply(m@object$GetValues(name), .fromComValue))

setMethod("show", signature("Metadata"), function(object)
{
	cat("Macrobond metadata\n")
	cat("Readonly :", getIsReadonly(object), "\n")
	cat("Values:\n")
	mn <- getMetadataNames(object)
	f <- data.frame(mn, Value = I(lapply(getMetadataNames(object)[,"Name"], function(mv) getFirstMetadataValue(object, mv))))
	show(f)
})

# MetadataInformation
MetadataInformation <- setClass("MetadataInformation", slots = c(object = "COMIDispatch"))

setMethod("getName", signature("MetadataInformation"), function(object) object@object[["Name"]] )

setGeneric("getDescription", function(m) standardGeneric("getDescription"))
setMethod("getDescription", signature("MetadataInformation"), function(m) m@object[["Description"]] )

setGeneric("getComment", function(m) standardGeneric("getComment"))
setMethod("getComment", signature("MetadataInformation"), function(m) m@object[["Comment"]] )

setGeneric("getUsesValueList", function(m) standardGeneric("getUsesValueList"))
setMethod("getUsesValueList", signature("MetadataInformation"), function(m) m@object[["UsesValueList"]] )

setGeneric("getValueType", function(m) standardGeneric("getValueType"))
setMethod("getValueType", signature("MetadataInformation"), function(m)
{
	vt <- m@object[["ValueType"]]
	if (vt == 3) "integer"
	else if (vt == 5) "numeric"
	else if (vt == 7) "POSIXlt"
	else if (vt == 8) "character"
	else if (vt == 11) "logical"
	else NULL
})

setGeneric("getCanHaveMultipleValues", function(m) standardGeneric("getCanHaveMultipleValues"))
setMethod("getCanHaveMultipleValues", signature("MetadataInformation"), function(m) m@object[["CanHaveMultipleValues"]] )

setGeneric("getIsDatabaseEntity", function(m) standardGeneric("getIsDatabaseEntity"))
setMethod("getIsDatabaseEntity", signature("MetadataInformation"), function(m) m@object[["IsDatabaseEntity"]] )

setGeneric("getCanListValues", function(m) standardGeneric("getCanListValues"))
setMethod("getCanListValues", signature("MetadataInformation"), function(m) m@object[["CanListValues"]] )

setGeneric("getValuePresentationText", function(m, value) standardGeneric("getValuePresentationText"))
setMethod("getValuePresentationText", signature("MetadataInformation", "ANY"), function(m, value) m@object$GetValuePresentationText(.toComValue(value)) )

setGeneric("getValueInformation", function(m, value) standardGeneric("getValueInformation"))
setMethod("getValueInformation", signature("MetadataInformation", "ANY"), function(m, value) { info <- m@object$GetValueInformation(value); vl = c(info[["Value"]], info[["Description"]], .nullAsEmptyString(info[["Comment"]])); names(vl) <- c("Name", "Description", "Comment"); vl } )

setGeneric("listAllValues", function(m) standardGeneric("listAllValues"))
setMethod("listAllValues", signature("MetadataInformation"), function(m)
{
	infos <- m@object$ListAllValues()
	values = lapply(infos, function(info) info[["Value"]])
	descriptions = lapply(infos, function(info) info[["Description"]])
	comments = lapply(infos, function(info) .nullAsEmptyString(info[["Comment"]]))
	vl = matrix(c(values, descriptions, comments), nrow = length(values), ncol = 3)
	colnames(vl) <- c("Name", "Description", "Comment")
	vl
} )

setMethod("show", signature("MetadataInformation"), function(object)
{
	cat("Macrobond metadata information\n")
	cat("Name :", getName(object), "\n")
	cat("Description :", getDescription(object), "\n")
	cat("Comment :", getComment(object), "\n")
	cat("Value type :", getValueType(object), "\n")
	cat("Uses value list :", getUsesValueList(object), "\n")
	cat("Can list values :", getCanListValues(object), "\n")
	cat("Can have multiple values :", getCanHaveMultipleValues(object), "\n")
})


# SeriesRequest

setGeneric("setFrequency", function(m, f) standardGeneric("setFrequency"))
setMethod("setFrequency", signature("SeriesRequest", "TimeSeriesFrequencyClass"), function(m, f) m@object[["Frequency"]] <- unclass(f) )
setMethod("setFrequency", signature("SeriesRequest", "character"), function(m, f) m@object[["Frequency"]] <- unclass(TimeSeriesFrequency[[f]]) )
setMethod("getFrequency", signature("SeriesRequest"), function(object) .findEnum(TimeSeriesFrequency, object@object[["Frequency"]], TimeSeriesFrequencyClass))

setGeneric("setWeekdays", function(m, weekdays) standardGeneric("setWeekdays"))
setMethod("setWeekdays", signature("SeriesRequest", "TimeSeriesWeekdaysClass"), function(m, weekdays) m@object[["Weekdays"]] <- unclass(weekdays) )
setMethod("setWeekdays", signature("SeriesRequest", "character"), function(m, weekdays) m@object[["Weekdays"]] <- unclass(TimeSeriesWeekdays[[weekdays]]) )
setMethod("getWeekdays", signature("SeriesRequest"), function(object) .findEnum(TimeSeriesWeekdays, object@object[["Weekdays"]], TimeSeriesWeekdaysClass))

setGeneric("setCurrency", function(m, currency) standardGeneric("setCurrency"))
setMethod("setCurrency", signature("SeriesRequest", "character"), function(m, currency) m@object[["Currency"]] <- currency )
setGeneric("getCurrency", function(m) standardGeneric("getCurrency"))
setMethod("getCurrency", signature("SeriesRequest"), function(m) m@object[["Currency"]])

setGeneric("setCalendarMergeMode", function(m, mergeMode) standardGeneric("setCalendarMergeMode"))
setMethod("setCalendarMergeMode", signature("SeriesRequest", "CalendarMergeModeClass"), function(m, mergeMode) m@object[["CalendarMergeMode"]] <- unclass(mergeMode) )
setMethod("setCalendarMergeMode", signature("SeriesRequest", "character"), function(m, mergeMode) m@object[["CalendarMergeMode"]] <- unclass(CalendarMergeMode[[mergeMode]]) )
setGeneric("getCalendarMergeMode", function(m) standardGeneric("getCalendarMergeMode"))
setMethod("getCalendarMergeMode", signature("SeriesRequest"), function(m) .findEnum(CalendarMergeMode, m@object[["CalendarMergeMode"]], CalendarMergeModeClass))

setGeneric("setStartDate", function(object, startDate) standardGeneric("setStartDate"))
setMethod("setStartDate", signature("SeriesRequest", "Date"), function(object, startDate) object@object[["StartDate"]] <- .toComValue(startDate) )
setMethod("setStartDate", signature("SeriesRequest", "character"), function(object, startDate) object@object[["StartDate"]] <- startDate )
setMethod("getStartDate", signature("SeriesRequest"), function(object) .fromComValueDate(object@object[["StartDate"]]) )

setGeneric("setStartDateMode", function(m, dateMode) standardGeneric("setStartDateMode"))
setMethod("setStartDateMode", signature("SeriesRequest", "CalendarDateModeClass"), function(m, dateMode) m@object[["StartDateMode"]] <- unclass(dateMode) )
setMethod("setStartDateMode", signature("SeriesRequest", "character"), function(m, dateMode) m@object[["StartDateMode"]] <- unclass(CalendarDateMode[[dateMode]]) )
setGeneric("getStartDateMode", function(m) standardGeneric("getStartDateMode"))
setMethod("getStartDateMode", signature("SeriesRequest"), function(m) .findEnum(CalendarDateMode, m@object[["StartDateMode"]], CalendarDateModeClass))

setGeneric("setEndDate", function(object, endDate) standardGeneric("setEndDate"))
setMethod("setEndDate", signature("SeriesRequest", "Date"), function(object, endDate) object@object[["EndDate"]] <- .toComValue(endDate) )
setMethod("setEndDate", signature("SeriesRequest", "character"), function(object, endDate) object@object[["EndDate"]] <- endDate )
setMethod("getEndDate", signature("SeriesRequest"), function(object) .fromComValueDate(object@object[["EndDate"]]) )

setGeneric("setEndDateMode", function(m, dateMode) standardGeneric("setEndDateMode"))
setMethod("setEndDateMode", signature("SeriesRequest", "CalendarDateModeClass"), function(m, dateMode) m@object[["EndDateMode"]] <- unclass(dateMode) )
setMethod("setEndDateMode", signature("SeriesRequest", "character"), function(m, dateMode) m@object[["EndDateMode"]] <- unclass(CalendarDateMode[[dateMode]]) )
setGeneric("getEndDateMode", function(m) standardGeneric("getEndDateMode"))
setMethod("getEndDateMode", signature("SeriesRequest"), function(m) .findEnum(CalendarDateMode, m@object[["EndDateMode"]], CalendarDateModeClass))

setGeneric("addSeries", function(m, name) standardGeneric("addSeries"))
setMethod("addSeries", signature("SeriesRequest", "character"), function(m, name) invisible(SeriesExpression(object = m@object$AddSeries(name))) )

setGeneric("getAddedSeries", function(m) standardGeneric("getAddedSeries"))
setMethod("getAddedSeries", signature("SeriesRequest"), function(m) lapply(m@object[["AddedSeries"]], function(s) SeriesExpression(object = s)))

setMethod("show", signature("SeriesRequest"), function(object)
{
	cat("Macrobond series request\n")
	cat("Frequency :", names(getFrequency(object)), "\n")
	cat("Weekdays :", names(getWeekdays(object)), "\n")
	cat("Currency :", getCurrency(object), "\n")
	cat("Calendar merge mode :", names(getCalendarMergeMode(object)), "\n")
	cat(paste("Start date : ", getStartDate(object), "\n"))
	cat("Start date mode :", names(getStartDateMode(object)), "\n")
	cat(paste("End date : ", getEndDate(object), "\n"))
	cat("End date mode :", names(getEndDateMode(object)), "\n")
	cat("\nRequested series:\n")
	show(getAddedSeries(object))
})

# SeriesExpression
SeriesExpression <- setClass("SeriesExpression", slots = c(object = "COMIDispatch"))

setGeneric("setName", function(object, name) standardGeneric("setName"))
setMethod("setName", signature("SeriesExpression", "character"), function(object, name) object@object[["Name"]] <- name )
setMethod("getName", signature("SeriesExpression"), function(object) object@object[["Name"]] )

setGeneric("setMissingValueMethod", function(m, method) standardGeneric("setMissingValueMethod"))
setMethod("setMissingValueMethod", signature("SeriesExpression", "TimeSeriesMissingValueMethodClass"), function(m, method) m@object[["MissingValueMethod"]] <- unclass(method) )
setMethod("setMissingValueMethod", signature("SeriesExpression", "character"), function(m, method) m@object[["MissingValueMethod"]] <- unclass(TimeSeriesMissingValueMethod[[method]]) )
setGeneric("getMissingValueMethod", function(m) standardGeneric("getMissingValueMethod"))
setMethod("getMissingValueMethod", signature("SeriesExpression"), function(m) .findEnum(TimeSeriesMissingValueMethod, m@object[["MissingValueMethod"]], TimeSeriesMissingValueMethodClass) )

setGeneric("setToLowerFrequencyMethod", function(m, method) standardGeneric("setToLowerFrequencyMethod"))
setMethod("setToLowerFrequencyMethod", signature("SeriesExpression", "TimeSeriesToLowerFrequencyMethodClass"), function(m, method) m@object[["ToLowerFrequencyMethod"]] <- unclass(method) )
setMethod("setToLowerFrequencyMethod", signature("SeriesExpression", "character"), function(m, method) m@object[["ToLowerFrequencyMethod"]] <- unclass(TimeSeriesToLowerFrequencyMethod[[method]]) )
setGeneric("getToLowerFrequencyMethod", function(m) standardGeneric("getToLowerFrequencyMethod"))
setMethod("getToLowerFrequencyMethod", signature("SeriesExpression"), function(m) .findEnum(TimeSeriesToLowerFrequencyMethod, m@object[["ToLowerFrequencyMethod"]], TimeSeriesToLowerFrequencyMethodClass) )

setGeneric("setToHigherFrequencyMethod", function(m, method) standardGeneric("setToHigherFrequencyMethod"))
setMethod("setToHigherFrequencyMethod", signature("SeriesExpression", "TimeSeriesToHigherFrequencyMethodClass"), function(m, method) m@object[["ToHigherFrequencyMethod"]] <- unclass(method) )
setMethod("setToHigherFrequencyMethod", signature("SeriesExpression", "character"), function(m, method) m@object[["ToHigherFrequencyMethod"]] <- unclass(TimeSeriesToHigherFrequencyMethod[[method]]) )
setGeneric("getToHigherFrequencyMethod", function(m) standardGeneric("getToHigherFrequencyMethod"))
setMethod("getToHigherFrequencyMethod", signature("SeriesExpression"), function(m) .findEnum(TimeSeriesToHigherFrequencyMethod, m@object[["ToHigherFrequencyMethod"]], TimeSeriesToHigherFrequencyMethodClass) )

setMethod("show", signature("SeriesExpression"), function(object)
{
	cat("Macrobond series expression\n")
	cat("Name :", getName(object), "\n")
	cat("Missing value method :", names(getMissingValueMethod(object)), "\n")
	cat("To lower frequency method :", names(getToLowerFrequencyMethod(object)), "\n")
	cat("To higher frequency method :", names(getToHigherFrequencyMethod(object)), "\n")
})

as.xts.TimeSeries <- function(x, ...) { s <- xts(getValues(x), order.by=getDatesAtStartOfPeriod(x), ...); colnames(s) <- getName(x); s }

MakeXtsFromUnifiedResponse <- function(listOfSeries)
{
	values <- cbind(sapply(listOfSeries, function(s) getValues(s)))
	xts(values, order.by=getDatesAtStartOfPeriod(listOfSeries[[1]]))
}