\name{MacrobondAPI-package}
\alias{MacrobondAPI-package}
\alias{MacrobondAPI}
\docType{package}
\title{\packageTitle{MacrobondAPI}}
\description{\packageDescription{MacrobondAPI}}
\details{
With the MacrobondAPI you can download time series from the Macrobond database. You can also upload in-house series.

The most important methods are \code{\link{FetchTimeSeries}} and \code{\link{FetchOneTimeSeries}}. These allows you to download time series along with metadata for the series.

The methods \code{\link{CreateUnifiedTimeSeriesRequest}} is used when you want to download and convert a set of time series to a common frequency and calendar. You can specify the frequency conversion method as well as the method to use for filling in any missing values. The series can be converted to a common currency. 

You can create new time series and upload them to the in-house database by using the method \code{\link{UploadOneOrMoreTimeSeries}}.

With the method \code{\link{getMetadata}} you can get \code{\linkS4class{Metadata}} for \code{\linkS4class{TimeSeries}} and \code{\linkS4class{Entity}} objects.

The method \code{\link{as.xts}} and \code{\link{MakeXtsFromUnifiedResponse}} converts Macrobond time series objects to \pkg{xts} objects.
}

\author{
\packageAuthor{MacrobondAPI}

Maintainer: \packageMaintainer{MacrobondAPI}
}

\references{
\url{https://help.macrobond.com/add-ins/},
\url{http://www.omegahat.net/RDCOMClient/}
}

\seealso{
	\code{\link{FetchOneTimeSeries}}
	\code{\link{FetchTimeSeries}}
	\code{\link{getMetadata}}
	\code{\link{CreateUnifiedTimeSeriesRequest}}
	\code{\link{UploadOneOrMoreTimeSeries}}
	\code{\link{as.xts.TimeSeries}}
}

\examples{
	# Load the series "usgdp". If you want to load several series you can use "FetchTimeSeries(listOfSeriesNames)" instead.
	seriesGdp <- FetchOneTimeSeries("usgdp")
	# Check that the series loaded OK
	if (getIsError(seriesGdp))
		stop(getErrorMessage(seriesGdp))

	# Show a summary of the series object (same as "show(seriesGdp)")
	seriesGdp

	# Show the title
	getTitle(seriesGdp)

	# Get the values and dates
	vectorOfValues <- getValues(seriesGdp)
	vectorOfObservationDates <- getDatesAtStartOfPeriod(seriesGdp)

	# Make a simple plot of the series
	plot(seriesGdp)

	# Get the currency of the series
	metaGdp <- getMetadata(seriesGdp)
	currency <- getFirstMetadataValue(metaGdp, "Currency")

	# Convert the Macrobond time series object to an xts series
	seriesGdp.xts <- as.xts(seriesGdp)

	# Upload an in-house time series
	seriesNew = CreateTimeSeriesObject("ih:mb:priv:s1", "My forecast", "us", "Forecasts", "Monthly", as.Date("1990-01-01"), c(12.2, 12.7, 12.8, 13.0))
	UploadOneOrMoreTimeSeries(seriesNew)

	# Get information about a release.
	# The name of the release is a metadata property of the time series.
	# The release has a corresponding "entity" in the database.
	releaseGdpName <- getFirstMetadataValue(metaGdp, "Release")
	if (!is.null(releaseGdpName))
	\{
	   releaseGdp <- FetchOneEntity(releaseGdpName)
	   # The NextReleaseEventTime attribute contains information about the next scheduled release of updated series.
	   # The date might be missing if no release date is known.
	   getFirstMetadataValue(getMetadata(releaseGdp), "NextReleaseEventTime")
	\}
}

\keyword{package}
\keyword{database}
