\name{Metadata-class}
\Rdversion{1.1}
\docType{class}
\alias{Metadata}
\alias{Metadata-class}
\alias{addMetadataValue}
\alias{addMetadataValue,Metadata,character-method}
\alias{getFirstMetadataValue}
\alias{getFirstMetadataValue,Metadata,character-method}
\alias{getIsReadonly}
\alias{getIsReadonly,Metadata-method}
\alias{getMetadataNames}
\alias{getMetadataNames,Metadata-method}
\alias{getMetadataValues}
\alias{getMetadataValues,Metadata,character-method}
\alias{hideMetadataValue}
\alias{hideMetadataValue,Metadata,character-method}
\alias{show,Metadata-method}

\title{Metadata object}
\description{
A Metadata object is a container for metadata attributes of a \code{\linkS4class{TimeSeries}} or \code{\linkS4class{Entity}}.
Metadata is represented by a name and a value. The type of the value can be a \code{\link{character}}, \code{\link{logical}}, \code{\link{numeric}}, \code{\link{integer}} or \code{\link{POSIXlt}}. An examples of a metadata names are "Region" and "Currency". Some metadata can have multiple values, like Region, but others, Currency, can have only one value.
You can find out more about each type of metadata attribute through the \code{\linkS4class{MetadataInformation}} class.
}

\section{Objects from the Class}{
Metadata for a \code{\linkS4class{TimeSeries}} or \code{\linkS4class{Entity}} can be retrieved by calling \code{\link{getMetadata}}.

New Metadata objects can be created by calling \code{\link{CreateEmptyMetadata}} or \code{\link{CreateDerivedMetadata}}. The content of newly created objects can be modified and can then be used when uploading in-house series.
}

\section{Methods}{
  \describe{
    \item{getFirstMetadataValue}{\code{signature(m = "Metadata", name = "character")}: Get the first metadata value for the attribute. This method is convenient since most metadata attributes can have only one value. If the attribute does not exist, then \code{\link{NULL}} is returned. }
    \item{getMetadataValues}{\code{signature(m = "Metadata", name = "character")}: Get a \code{\link{list}} of all the metadata value for the attribute. If the attribute does not exist, then an empty \code{\link{list}} is returned. }
    \item{addMetadataValue}{\code{signature(m = "Metadata", name = "character", value = "ANY")}: Add a metadata value of the specified name. The value can be a single value or a list, if that is allowed for the type of metadata. The data type of the value must match the definition in \code{\linkS4class{MetadataInformation}}. }
    \item{getIsReadonly}{\code{signature(m = "Metadata")}: Indicating whether new values can be added or hidden in this instance. Typically, only newly created instances of the metadata objects can be edited and only until they have been used in a series. }
    \item{getMetadataNames}{\code{signature(m = "Metadata")}: Get a list of all the metadata specified together with a description for each name. }
    \item{hideMetadataValue}{\code{signature(m = "Metadata", name = "character")}: Hide a value inherited from a parent metadata object. }
    \item{show}{\code{signature(object = "Metadata")}: Show a list of all the attributes in the object. }
	 }
}

\details{
Commonly used meta data attribute names are:
\describe{
    \item{Region}{A region code based on the ISO 3166 standard.}
    \item{Currency}{The currency code based on the three letter ISO 4217 codes.}
    \item{DisplayUnit}{The unit of the series.}
    \item{Class}{The class is a string with one of the values "stock" or "flow". This determines how the automatic frequency conversion in Macrobond works. If the value is flow, the data will be aggregated when converted to a lower frequency and distributed over the period when converted to a higher frequency.}
    \item{MaturityDate}{Some analyses, such as the Yield curve analysis, can use this information to automatically configure the maturity length. If you set this value, you should also set RateMethod but not set the MaturityDays value.}
    \item{MaturityDays}{This value is a positive integer that some analyses, such as the Yield curve analysis, can use this information to automatically configure the maturity length. If you set this value, you should also set RateMethod but not set the MaturityDate value. 1 week is 7 days, 1 month is 30 days and one year is 360 days.}
    \item{RateMethod}{This value should be either "simple" or "effective" that some analyses, such as the Yield curve analysis, can use this information for automatically configuration. If you set this value, you should also set one of the MaturityDate or MaturityLength values.}
    \item{ForecastCutoffDate}{This is used only when creating new in-house series. All observation at this date and later will flagged as forecasts. The date must not be before the start or after the end of the series.}
    \item{Currency}{The currency code based on the three letter ISO 4217 codes.}
    \item{LastModifiedTimeStamp}{When the entity was last modified. This includes changes in both values (for time series) as well as metadata.}

    \item{Source}{The name of the source entity. Use this name in FetchOneEntity in order to get more information about the source.}
    \item{Security}{The name of the security entity. Use this name in FetchOneEntity in order to get more information about the security.}
    \item{Company}{The name of the company entity. Use this name in FetchOneEntity in order to get more information about the company.}

    \item{Release}{The name of the release entity. Use this name in FetchOneEntity in order to get more information about the release.}
    \item{LastReleaseEventTime}{For entities of type "Release": A timestamp for the previous release.}
    \item{NextReleaseEventTime}{For entities of type "Release": A timestamp for the next scheduled release.}
	 }
}

\seealso{
\code{\link{getMetadata}},
\code{\link{CreateEmptyMetadata}},
\code{\link{CreateDerivedMetadata}},
\code{\link{UploadOneOrMoreTimeSeries}},
\code{\linkS4class{MetadataInformation}}
}

\examples{
seriesGdp <- FetchOneTimeSeries("usgdp")
meta <- getMetadata(seriesGdp)
show(meta)
}

\keyword{classes}
