\name{SeriesExpression-class}
\Rdversion{1.1}
\docType{class}
\alias{SeriesExpression-class}
\alias{SeriesExpression}
\alias{getMissingValueMethod}
\alias{getMissingValueMethod,SeriesExpression-method}
\alias{getName,SeriesExpression-method}
\alias{getToHigherFrequencyMethod}
\alias{getToHigherFrequencyMethod,SeriesExpression-method}
\alias{getToLowerFrequencyMethod}
\alias{getToLowerFrequencyMethod,SeriesExpression-method}
\alias{setMissingValueMethod}
\alias{setMissingValueMethod,SeriesExpression,character-method}
\alias{setMissingValueMethod,SeriesExpression,TimeSeriesMissingValueMethodClass-method}
\alias{setName}
\alias{setName,SeriesExpression,character-method}
\alias{setToHigherFrequencyMethod}
\alias{setToHigherFrequencyMethod,SeriesExpression,character-method}
\alias{setToHigherFrequencyMethod,SeriesExpression,TimeSeriesToHigherFrequencyMethodClass-method}
\alias{setToLowerFrequencyMethod}
\alias{setToLowerFrequencyMethod,SeriesExpression,character-method}
\alias{setToLowerFrequencyMethod,SeriesExpression,TimeSeriesToLowerFrequencyMethodClass-method}
\alias{show,SeriesExpression-method}

\title{SeriesExpression class}

\description{
	Information about series in a unified series request.
	}

\section{Objects from the Class}{
	You create an instance through a call to \code{\link{addSeries}}.
	The default values of the properties are:

	\tabular{ll}{
		MissingValueMethod \tab "Auto"\cr
		ToHigherFrequencyMethod \tab "Auto"\cr
		ToLowerFrequencyMethod \tab "Auto"\cr
		}

	}

\section{Methods}{
  \describe{
    \item{getMissingValueMethod}{\code{signature(m = "SeriesExpression")}: Get the method used for filling in any missing values. It will be one of the values in the list \code{\link{TimeSeriesMissingValueMethod}}. }
    \item{setMissingValueMethod}{\code{signature(m = "SeriesExpression", method = "TimeSeriesMissingValueMethodClass")}: Set the method used for filling in missing values. It should be one of the values in the list \code{\link{TimeSeriesMissingValueMethod}}. }
    \item{setMissingValueMethod}{\code{signature(m = "SeriesExpression", method = "character")}: Set the method used for filling in missing values. It should be one of the names of the list \code{\link{TimeSeriesMissingValueMethod}}. }

    \item{getName}{\code{signature(object = "SeriesExpression")}: Get the name of the series. }
    \item{setName}{\code{signature(object = "SeriesExpression", name = "character")}: Set the name of the series. }

    \item{getToHigherFrequencyMethod}{\code{signature(m = "SeriesExpression")}: Get the method used for converting to a higher frequency. It will be one of the values in the list \code{\link{TimeSeriesToHigherFrequencyMethod}}. }
	\item{setToHigherFrequencyMethod}{\code{signature(m = "SeriesExpression", method = "TimeSeriesToHigherFrequencyMethodClass")}: Set the method used for converting to a higher frequency. It should be one of the values in the list \code{\link{TimeSeriesToHigherFrequencyMethod}}. }
    \item{setToHigherFrequencyMethod}{\code{signature(m = "SeriesExpression", method = "character")}: Set the method used for converting to a higher frequency. It will be be one of the names of the list \code{\link{TimeSeriesToHigherFrequencyMethod}}. }
    
    \item{getToLowerFrequencyMethod}{\code{signature(m = "SeriesExpression")}: Get the method used for converting to a lower frequency. It will be one of the values in the list \code{\link{TimeSeriesToLowerFrequencyMethod}}. }
    \item{setToLowerFrequencyMethod}{\code{signature(m = "SeriesExpression", method = "TimeSeriesToLowerFrequencyMethodClass")}: Set the method used for converting to a lower frequency. It should be one of the values in the list \code{\link{TimeSeriesToLowerFrequencyMethod}}. }
    \item{setToLowerFrequencyMethod}{\code{signature(m = "SeriesExpression", method = "character")}: Set the method used for converting to a lower frequency. It will be one of the names of the list \code{\link{TimeSeriesToLowerFrequencyMethod}}. }

    \item{show}{\code{signature(object = "SeriesExpression")}: Show a summary of the time series expression. }
	 }
}

\seealso{
	\code{\linkS4class{SeriesRequest}}
	\code{\link{addSeries}}
	\code{\link{CreateUnifiedTimeSeriesRequest}}
	\code{\link{FetchTimeSeries}}
}

\examples{
seriesRequest <- CreateUnifiedTimeSeriesRequest()
setFrequency(seriesRequest, TimeSeriesFrequency[["Monthly"]])
addSeries(seriesRequest, "usgdp")
seriesExpressionCpi <- addSeries(seriesRequest, "uscpi")
setMissingValueMethod(seriesExpressionCpi, "None")
twoSeries <- FetchTimeSeries(seriesRequest)
}

\keyword{classes}
