\name{SeriesRequest-class}
\Rdversion{1.1}
\docType{class}
\alias{SeriesRequest-class}
\alias{SeriesRequest}
\alias{addSeries}
\alias{addSeries,SeriesRequest,character-method}
\alias{FetchTimeSeries,SeriesRequest-method}
\alias{getAddedSeries}
\alias{getAddedSeries,SeriesRequest-method}
\alias{getCalendarMergeMode}
\alias{getCalendarMergeMode,SeriesRequest-method}
\alias{getCurrency}
\alias{getCurrency,SeriesRequest-method}
\alias{getEndDate,SeriesRequest-method}
\alias{getEndDateMode}
\alias{getEndDateMode,SeriesRequest-method}
\alias{getFrequency,SeriesRequest-method}
\alias{getStartDate,SeriesRequest-method}
\alias{getStartDateMode}
\alias{getStartDateMode,SeriesRequest-method}
\alias{getWeekdays,SeriesRequest-method}
\alias{setCalendarMergeMode}
\alias{setCalendarMergeMode,SeriesRequest,CalendarMergeModeClass-method}
\alias{setCalendarMergeMode,SeriesRequest,character-method}
\alias{setCurrency}
\alias{setCurrency,SeriesRequest,character-method}
\alias{setEndDate}
\alias{setEndDate,SeriesRequest,character-method}
\alias{setEndDate,SeriesRequest,Date-method}
\alias{setEndDateMode}
\alias{setEndDateMode,SeriesRequest,CalendarDateModeClass-method}
\alias{setEndDateMode,SeriesRequest,character-method}
\alias{setFrequency}
\alias{setFrequency,SeriesRequest,character-method}
\alias{setFrequency,SeriesRequest,TimeSeriesFrequencyClass-method}
\alias{setStartDate}
\alias{setStartDate,SeriesRequest,character-method}
\alias{setStartDate,SeriesRequest,Date-method}
\alias{setStartDateMode}
\alias{setStartDateMode,SeriesRequest,CalendarDateModeClass-method}
\alias{setStartDateMode,SeriesRequest,character-method}
\alias{setWeekdays}
\alias{setWeekdays,SeriesRequest,character-method}
\alias{setWeekdays,SeriesRequest,TimeSeriesWeekdaysClass-method}
\alias{show,SeriesRequest-method}

\title{SeriesRequest class}

\description{
	Use this class to set up a request of unified series. Once the parameters are configured, the object is passed to \code{\link{FetchTimeSeries}} in order to download and process the time series.

	Default values of properties:

	\tabular{ll}{
		Frequency \tab "Highest"\cr
		Currency \tab "" (No conversion)\cr
		StartDate \tab "" (No limit)\cr
		StartDateMode \tab "DataInAnySeries"\cr
		EndDate \tab "" (No limit)\cr
		EndDateMode \tab "DataInAnySeries"\cr
		CalendarMergeMode \tab "AvailableInAny"
		}
	}

\section{Objects from the Class}{
	Objects are created by calling \code{\link{CreateUnifiedTimeSeriesRequest}}.
	}

\section{Methods}{
  \describe{
    \item{addSeries}{\code{signature(m = "SeriesRequest", name = "character")}:  Add a series to a uniform series request. It returns a value of type \code{\linkS4class{SeriesExpression}}, which can be used to set further properties. }

    \item{FetchTimeSeries}{\code{signature(request = "SeriesRequest")}: Download one or more series from the database.
		The parameter can be a string, a vector of series names or an object created by \code{\link{CreateUnifiedTimeSeriesRequest}}.
		The result is a vector of series in the same order as requested.
		}
    \item{getAddedSeries}{\code{signature(m = "SeriesRequest")}: Get a list of the series added to the request. The elements are of type \code{\linkS4class{SeriesExpression}}. }

    \item{getFrequency}{\code{signature(object = "SeriesRequest")}: Set the frequency as a value from the list \code{\link{TimeSeriesFrequency}}. }
    \item{setFrequency}{\code{signature(m = "SeriesRequest", f = "TimeSeriesFrequencyClass")}: value from the list \code{\link{TimeSeriesFrequency}}. }
    \item{setFrequency}{\code{signature(m = "SeriesRequest", f = "character")}: Set the frequency for a unified series request with a name from the list \code{\link{TimeSeriesFrequency}}. }

    \item{getWeekdays}{\code{signature(object = "SeriesRequest")}: Get a bit set of what weekdays that are used for daily timeseries. The bits are defined for Monday-Sunday in the list \code{\link{TimeSeriesWeekdays}}. }
    \item{setWeekdays}{\code{signature(m = "SeriesRequest", weekdays = "TimeSeriesWeekdaysClass")}: Sets what days to include for a daily series. Typical values can be found in the list \code{\link{TimeSeriesWeekdays}}. }
    \item{setWeekdays}{\code{signature(m = "SeriesRequest", weekdays = "character")}: Set the name of the weekday pattern to use for daily series from the list \code{\link{TimeSeriesWeekdays}}. }

    \item{getCurrency}{\code{signature(m = "SeriesRequest")}: Get the currency code used for currency conversion. An empty string, which is the default value, means that no conversion will take place. }
    \item{setCurrency}{\code{signature(m = "SeriesRequest", currency = "character")}: Set the currency code, based on the three letter ISO 4217 codes, that will be used to convert any series expressed in currency units. The default is an empty string, which means that no currency conversion will be done. A list of the supported currencies can be found here: \href{https://www.macrobond.com/go/currencyList}{List of currency codes}. }

    \item{getStartDate}{\code{signature(object = "SeriesRequest")}: Set the start date used for all the series in the request.
		The value can be an empty string, a specific date or a string representing a point in time as described below.
		If it is empty or a relative reference, the StartDateMode will be used to determine the end.
		}
    \item{setStartDate}{\code{signature(object = "SeriesRequest", startDate = "Date")}: From and including the specified date. }
    \item{setStartDate}{\code{signature(object = "SeriesRequest", startDate = "character")}: Set a relative start date.
		}

    \item{getStartDateMode}{\code{signature(m = "SeriesRequest")}: Determines if the automatic start of the series is at the last point when there is data in any series or the last point where there is data in all series.
		This setting is not used when the StartDate property is set to an absolute point in time. The default is "DataInAnySeries".
		}
    \item{setStartDateMode}{\code{signature(m = "SeriesRequest", dateMode = "CalendarDateModeClass")}: Set the mode with a value from the list \code{\link{CalendarDateMode}}. }
    \item{setStartDateMode}{\code{signature(m = "SeriesRequest", dateMode = "character")}: Set the mode with a name from the list \code{\link{CalendarDateMode}}. }

    \item{getEndDate}{\code{signature(object = "SeriesRequest")}: This specifies the end date used for all the series in the request. The value can be empty, a specific date or a string representing a point in time as described below. If it is empty or a relative reference, the EndDateMode will be used to determine the end. }
    \item{setEndDate}{\code{signature(object = "SeriesRequest", endDate = "Date")}: Until and including the specified date. }
    \item{setEndDate}{\code{signature(object = "SeriesRequest", endDate = "character")}: Set a relative end date.
		}

    \item{getEndDateMode}{\code{signature(m = "SeriesRequest")}: Determines if the automatic end of the series is at the last point when there is data in any series or the last point where there is data in all series. The default is "DataInAnySeries". This setting is not used when the EndDate property is set to an absolute point in time. }
    \item{setEndDateMode}{\code{signature(m = "SeriesRequest", dateMode = "CalendarDateModeClass")}: Set the mode with a value from the list \code{\link{CalendarDateMode}}. }
    \item{setEndDateMode}{\code{signature(m = "SeriesRequest", dateMode = "character")}: Set the mode with a name from the list \code{\link{CalendarDateMode}}. }

    \item{getCalendarMergeMode}{\code{signature(m = "SeriesRequest")}: Returns the setting how to handle points in time that are not available in one or more series. It returns a value from the list \code{\link{CalendarMergeMode}}. }
    \item{setCalendarMergeMode}{\code{signature(m = "SeriesRequest", mergeMode = "CalendarMergeModeClass")}: Set how to handle points in time that are not available in one or more series. The value should be from the list \code{\link{CalendarMergeMode}}. }
    \item{setCalendarMergeMode}{\code{signature(m = "SeriesRequest", mergeMode = "character")}: Set how to handle points in time that are not available in one or more series. The value should be a name from the list \code{\link{CalendarMergeMode}}. }

    \item{show}{\code{signature(object = "SeriesRequest")}: Show a summary of the time series request. }
	 }
}

\note{
    \bold{Relative StartDate}
	The relative references, starting with "-", will be affected by the EndDateMode.
	\tabular{ll}{
		"" \tab The reference date depending on the StartDateMode property. This is the default value.\cr
		"1995" \tab The start of 1995.\cr
		"1995-05" \tab The start of May 1995.\cr
		"1995-05-01" \tab From and uncluding 1 May 1995.\cr
		"-50" \tab The last fifty observations.\cr
		"-2y" \tab The observations of the two last years.\cr
		"-1q" \tab The observations of the three months.\cr
		"-3w" \tab The observations of the three last weeks.\cr
		"-5d" \tab The observations of the five last days.
		}

    \bold{Relative EndDate}
	The EndDateMode property determines the reference point for the relative date.
	\tabular{ll}{
		"" \tab The reference date. This is the default value.\cr
		"1995" \tab The end of 1995\cr
		"1995-05" \tab The end of May 1995\cr
		"1995-05-01" \tab Until and uncluding 1 May 1995\cr
		"-50" \tab Fifty observations before the reference date\cr
		"-2y" \tab Two years before the reference date\cr
		"-1q" \tab One quarter before the reference date\cr
		"-3w" \tab Three weeks before the reference date\cr
		"-5d" \tab Five days before the reference date
		}
}

\seealso{
\code{\link{CreateUnifiedTimeSeriesRequest}},
\code{\linkS4class{SeriesExpression}}
}

\examples{
seriesRequest <- CreateUnifiedTimeSeriesRequest()
setFrequency(seriesRequest, TimeSeriesFrequency[["Monthly"]])
addSeries(seriesRequest, "usgdp")
seriesExpressionCpi <- addSeries(seriesRequest, "uscpi")
setMissingValueMethod(seriesExpressionCpi, TimeSeriesMissingValueMethod[["None"]])
twoSeries <- FetchTimeSeries(seriesRequest)}

\keyword{classes}
