\name{TimeSeries-class}
\Rdversion{1.1}
\docType{class}
\alias{TimeSeries-class}
\alias{TimeSeries}
\alias{getDatesAtEndOfPeriod}
\alias{getDatesAtEndOfPeriod,TimeSeries-method}
\alias{getDatesAtStartOfPeriod}
\alias{getDatesAtStartOfPeriod,TimeSeries-method}
\alias{getEndDate}
\alias{getEndDate,TimeSeries-method}
\alias{getForecastFlags}
\alias{getForecastFlags,TimeSeries-method}
\alias{getFrequency}
\alias{getFrequency,TimeSeries-method}
\alias{getIndexAtDate}
\alias{getIndexAtDate,TimeSeries,Date-method}
\alias{getValueAtDate}
\alias{getValueAtDate,TimeSeries,Date-method}
\alias{getStartDate}
\alias{getStartDate,TimeSeries-method}
\alias{getTypicalObservationCountPerYear}
\alias{getTypicalObservationCountPerYear,TimeSeries-method}
\alias{getValues}
\alias{getValues,TimeSeries-method}
\alias{getWeekdays}
\alias{getWeekdays,TimeSeries-method}
\alias{plot.TimeSeries}
\alias{plot.TimeSeries,TimeSeries-method}
\alias{show,TimeSeries-method}

\title{TimeSeries class}

\description{
A TimeSeries object represents a time series downloaded via the Macrobond API or a time series that is to be uploaded to the Macrobond in-house database.
}

\section{Objects from the Class}{
Time series objects are created by the following methods:
\code{\link{FetchOneTimeSeries}}
\code{\link{FetchTimeSeries}}
\code{\link{CreateTimeSeriesObject}}
\code{\link{CreateTimeSeriesObjectWithForecastFlags}}
\code{\link{UploadOneOrMoreTimeSeries}}
}

\section{Extends}{
Class \code{"\linkS4class{Entity}"}, directly.
}

\section{Methods}{
  \describe{
    \item{getName}{\code{signature(object = "Entity")}: Get the name of the time series. This will always be the name used in the request. }
    \item{getPrimaryName}{\code{signature(object = "Entity")}: Get the primary name of the time series. This can differ from the name used in the request when that name is a alias or relative reference. }
    \item{getTitle}{\code{signature(series = "Entity")}: Get the title of the time series. }
    \item{getIsError}{\code{signature(series = "Entity")}: TRUE if there was an error downloading this time series. In that case the getErrorMessage method will return an error message. }
    \item{getErrorMessage}{\code{signature(series = "Entity")}: Get an error message text if getIsError is TRUE. }
    \item{getMetadata}{\code{signature(series = "Entity")}: Get a \code{\linkS4class{Metadata}} object for this time series. }

    \item{getValues}{\code{signature(series = "TimeSeries")}: Get a vector of all the values of the time series. }
    \item{getDatesAtStartOfPeriod}{\code{signature(series = "TimeSeries")}: Get a vector containg the \code{\link{Date}} at the start of each period. }
    \item{getDatesAtEndOfPeriod}{\code{signature(series = "TimeSeries")}: Get a vector containg the \code{\link{Date}} at the end of each period.}
    \item{getForecastFlags}{\code{signature(series = "TimeSeries")}: Get a vector with a \code{\link{logical}} for each value that tells if the value is a forecast or not.}

    \item{getStartDate}{\code{signature(object = "TimeSeries")}: Get the start date of the time series. }
    \item{getEndDate}{\code{signature(object = "TimeSeries")}: Get the end date of the time series. }
    \item{getFrequency}{\code{signature(object = "TimeSeries")}: Get the frequency of the time series as one of the values in \code{\link{TimeSeriesFrequency}}. }
    \item{getWeekdays}{\code{signature(object = "TimeSeries")}: Get a bit set of what weekdays that are used for daily timeseries. The bits are defined for Monday-Sunday in the list \code{\link{TimeSeriesWeekdays}}. }

    \item{getValueAtDate}{\code{signature(series = "TimeSeries", date = "Date")}: Get the value at or preceding a specific date.}
    \item{getIndexAtDate}{\code{signature(series = "TimeSeries", date = "Date")}:  Get the index in the value and date vectors at or preceding a specific date.}
    \item{getTypicalObservationCountPerYear}{\code{signature(series = "TimeSeries")}: The typical number of observations per year based on the frequency of the series. }
    \item{show}{\code{signature(object = "TimeSeries")}: Show a summary of the time series. }
    \item{plot.TimeSeries}{\code{signature(x = "TimeSeries")}: Show a simple plot of the time series. Additional parameters will be passed on to the \code{link[graphics]{plot}} function.}
	 }
}

\seealso{
\code{\link{FetchOneTimeSeries}},
\code{\link{FetchTimeSeries}},
\code{\link{CreateTimeSeriesObject}},
\code{\link{CreateTimeSeriesObjectWithForecastFlags}},
\code{\link{UploadOneOrMoreTimeSeries}}
}

\examples{
seriesGdp <- FetchOneTimeSeries("usgdp")
show(seriesGdp)
}
\keyword{classes}
